# Database drivers

import slogan
__all__= slogan.autoPackage(__file__)

import string

# A database driver sits between Slogan and whatever database module is being
# used, providing a standard interface. This would of course not be necessary
# if all database modules corresponded to DBI/1.0 and all database managers
# corresponded to SQL92, and does rather lead to the insane situation of Slogan
# talking to a database driver talking to a database module talking to an ODBC
# driver talking to a socket talking to the database server. But such is life,
# eh kids.

# Database driver interface:
#   identity
#     Short id string of the driver
#   connect(host, database, user, password)
#     Take whichever of the parameters are relevant and make a connection to
#     the database.
#   disconnect()
#     Close the connection.
#   command(sql)
#     Perform an SQL command on the open connection. No results returned.
#   query(sql)
#     Perform an SQL query on the open connection; return a list of results,
#     each a dictionary { [column name]: [value] }.
#   sqlLiteral(value)
#     Take a string value and convert it into a string literal suitable for
#     inclusion in an SQL query string.
#   sqlTextType
#     The name of the type for string columns.

# sqlLiteral and sql*Type are necessary for non-SQL92 DBMSs.

# Here are some generic database driver superclasses to build on now.

# Driver - base class for all database drivers

class Driver:
  identity= ''

  def __init__(self):
    pass
  def __del__(self):
    self.disconnect()

  def connect(self, host, database, user, password):
    pass
  def disconnect(self):
    pass

  def command(self, sql):
    pass
  def query(self, sql):
    return {}

  def sqlLiteral(self, value):
    return "'"+string.replace(value, "'", "''")+"'"
  sqlTextType= 'CHARACTER VARYING(%s) NOT NULL'

# Dbi1Driver - superclass for database modules supporting Python DBI 1.0

class Dbi1Driver(Driver):
  def disconnect(self):
    self.connection.close()

  def command(self, sql):
    cursor= self.connection.cursor()
    cursor.execute(sql)
    cursor.close()
  def query(self, sql):
    cursor= self.connection.cursor()
    cursor.execute(sql)
    results= []
    for row in cursor.fetchall():
      dict= {}
      for i in range(len(row)):
        dict[cursor.description[i][0]]= row[i]
      results.append(dict)
    cursor.close()
    return results

# Dbi2Driver - superclass for database modules supporting Python DBI 2.0

class Dbi2Driver(Dbi1Driver):
  def connect(self, host, database, user, password):
    self.connection= self.module.connect(user= user, password= password,
                                         host= host, database= database)

# BackslashModifier

# This is a superclass that can be applied to a Driver to change its SQL format
# to the MySQL/PostgreSQL "blackslash-is-an-escape-character" style.

class BackslashModifier:
  def sqlLiteral(self, value):
    return "'"+string.replace(string.replace(value,'\\','\\\\'), "'", "''")+"'"

# MySQLModifier

# MySQL also must use the BINARY directive to achieve case-sensitive matching

class MySQLModifier(BackslashModifier):
  sqlTextType= 'CHARACTER VARYING(%s) BINARY NOT NULL'
